@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2014 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2016 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -9,17 +9,15 @@
 #ifndef SQUID_TESTS_CAPTURINGSTORE_ENTRY_H
 #define SQUID_TESTS_CAPTURINGSTORE_ENTRY_H
 
-#include "Mem.h"
 #include "Store.h"
 
 /* class that captures various call data for test analysis */
 
 class CapturingStoreEntry : public StoreEntry
 {
-
-public:
     MEMPROXY_CLASS(CapturingStoreEntry);
 
+public:
     CapturingStoreEntry() : _buffer_calls(0), _flush_calls(0) {}
 
     String _appended_text;
@@ -35,11 +33,11 @@ public:
     }
 
     virtual void append(char const * buf, int len) {
+        if (!buf || len < 0) // old 'String' can't handle these cases
+            return;
         _appended_text.append(buf, len);
     }
 };
 
-MEMPROXY_CLASS_INLINE(CapturingStoreEntry);
-
 #endif
 
